use std::{
    io::{self, Read, Write},
    net::TcpStream,
};

#[cfg(feature = "https")]
use rustls::{ServerConnection, StreamOwned};

pub(crate) enum Stream {
    Tcp(TcpStream),
    #[cfg(feature = "https")]
    Tls(StreamOwned<ServerConnection, TcpStream>),
}

pub struct Client {
    stream: Stream,
}

impl Client {
    pub(crate) fn new(stream: Stream) -> Self {
        Client { stream }
    }

    // basically an alias for write (might remove)
    pub(crate) fn write_bytes(&mut self, mut bytes: Vec<u8>) -> io::Result<usize> {
        bytes.extend(b"\r\n");
        self.write(&bytes)
    }

    pub(crate) fn write_string(&mut self, mut string: String) -> io::Result<usize> {
        string.push_str("\r\n");
        self.write(string.as_bytes())
    }
}

impl Read for Client {
    fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
        match &mut self.stream {
            Stream::Tcp(stream) => stream.read(buf),
            #[cfg(feature = "https")]
            Stream::Tls(stream) => stream.read(buf),
        }
    }
}

impl Write for Client {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        match &mut self.stream {
            Stream::Tcp(stream) => stream.write(buf),
            #[cfg(feature = "https")]
            Stream::Tls(stream) => stream.write(buf),
        }
    }

    fn flush(&mut self) -> io::Result<()> {
        match &mut self.stream {
            Stream::Tcp(stream) => stream.flush(),
            #[cfg(feature = "https")]
            Stream::Tls(stream) => stream.flush(),
        }
    }
}
